@@ -172,7 +172,7 @@ QStringList ItemDataLoader::formatsToSave() const
 {
     return m_settings.contains("formats")
             ? m_settings["formats"].toStringList()
-            : QStringList() << mimeUriList << QString("text/xml");
+            : QStringList("text/xml");
 }
 
 QVariantMap ItemDataLoader::applySettings()
@@ -205,5 +205,3 @@ void ItemDataLoader::on_treeWidgetFormats_itemActivated(QTreeWidgetItem *item, i
     if ( !mime.isEmpty() )
         ui->plainTextEditFormats->appendPlainText(mime);
 }
-
-Q_EXPORT_PLUGIN2(itemdata, ItemDataLoader)
