@@ -55,12 +55,9 @@ QWidget *createIconWidget(const QByteArray &icon, QWidget *parent)
     if (!icon.isEmpty()) {
         QPixmap p;
         if (p.loadFromData(icon)) {
-#if QT_VERSION < 0x050000
-            const auto ratio = 1;
-#else
             const auto ratio = parent->devicePixelRatio();
             p.setDevicePixelRatio(ratio);
-#endif
+
             const int side = ratio * (iconFontSizePixels() + 2);
             p = p.scaled(side, side, Qt::KeepAspectRatio, Qt::SmoothTransformation);
             QLabel *label = new QLabel(parent);
@@ -355,5 +352,3 @@ bool ItemNotesLoader::matches(const QModelIndex &index, const QRegExp &re) const
     const QString text = index.data(contentType::notes).toString();
     return re.indexIn(text) != -1;
 }
-
-Q_EXPORT_PLUGIN2(itemnotes, ItemNotesLoader)
