@@ -87,6 +87,8 @@ bool readConfigHeader(QDataStream *stream)
 bool readConfig(QIODevice *file, QVariantMap *config)
 {
     QDataStream stream(file);
+    stream.setVersion(QDataStream::Qt_4_7);
+
     if ( !readConfigHeader(&stream) )
         return false;
 
@@ -108,15 +110,6 @@ void writeConfiguration(QIODevice *file, const QStringList &savedFiles)
     stream << config;
 }
 
-void setHeaderSectionResizeMode(QHeaderView *header, int logicalIndex, QHeaderView::ResizeMode mode)
-{
-#if QT_VERSION < 0x050000
-    header->setResizeMode(logicalIndex, mode);
-#else
-    header->setSectionResizeMode(logicalIndex, mode);
-#endif
-}
-
 QString iconFromId(int id)
 {
     return id != -1 ? QString(QChar(id)) : QString();
@@ -338,8 +331,8 @@ void fixUserMimeType(QString *mimeType)
 void setNormalStretchFixedColumns(QTableWidget *table, int normalColumn, int stretchColumn, int fixedColumn)
 {
     QHeaderView *header = table->horizontalHeader();
-    setHeaderSectionResizeMode(header, stretchColumn, QHeaderView::Stretch);
-    setHeaderSectionResizeMode(header, fixedColumn, QHeaderView::Fixed);
+    header->setSectionResizeMode(stretchColumn, QHeaderView::Stretch);
+    header->setSectionResizeMode(fixedColumn, QHeaderView::Fixed);
     header->resizeSection(fixedColumn, table->rowHeight(0));
     table->resizeColumnToContents(normalColumn);
 }
@@ -752,6 +745,7 @@ QWidget *ItemSyncLoader::createSettingsWidget(QWidget *parent)
 bool ItemSyncLoader::canLoadItems(QIODevice *file) const
 {
     QDataStream stream(file);
+    stream.setVersion(QDataStream::Qt_4_7);
     return readConfigHeader(&stream);
 }
 
@@ -869,5 +863,3 @@ ItemSaverPtr ItemSyncLoader::loadItems(const QString &tabName, QAbstractItemMode
 
     return std::make_shared<ItemSyncSaver>(model, tabPath, dir.path(), files, maxItems, m_formatSettings);
 }
-
-Q_EXPORT_PLUGIN2(itemsync, ItemSyncLoader)
