@@ -31,22 +31,22 @@
 
 #define QXTGLOBALSHORTCUT_H
 
-#include "qxtglobal.h"
 #include <QObject>
 #include <QKeySequence>
+
 class QxtGlobalShortcutPrivate;
 
-class QXT_GUI_EXPORT QxtGlobalShortcut : public QObject
+class QxtGlobalShortcut : public QObject
 {
     Q_OBJECT
-    QXT_DECLARE_PRIVATE(QxtGlobalShortcut)
     Q_PROPERTY(bool enabled READ isEnabled WRITE setEnabled)
+    Q_PROPERTY(bool valid READ isValid)
     Q_PROPERTY(QKeySequence shortcut READ shortcut WRITE setShortcut)
 
 public:
     explicit QxtGlobalShortcut(QObject* parent = nullptr);
     explicit QxtGlobalShortcut(const QKeySequence& shortcut, QObject* parent = nullptr);
-    virtual ~QxtGlobalShortcut();
+    ~QxtGlobalShortcut();
 
     QKeySequence shortcut() const;
     bool setShortcut(const QKeySequence& shortcut);
@@ -61,6 +61,10 @@ public Q_SLOTS:
 
 Q_SIGNALS:
     void activated(QxtGlobalShortcut *self);
+
+private:
+    friend class QxtGlobalShortcutPrivate;
+    QxtGlobalShortcutPrivate *d_ptr;
 };
 
 #endif // QXTGLOBALSHORTCUT_H
