@@ -53,25 +53,29 @@ class QxtGlobalShortcut {};
 
 namespace {
 
-bool hasScriptCommand(const QVector<Command> &commands)
+uint scriptCommandsHash(const QVector<Command> &commands)
 {
+    uint hash = 0;
+
     for (const auto &command : commands) {
-        if (command.type() == CommandType::Script)
-            return true;
+        if (command.type() == CommandType::Script && command.enable)
+            hash ^= qHash(command.cmd);
     }
 
-    return false;
+    return hash;
 }
 
 } // namespace
 
 ClipboardServer::ClipboardServer(QApplication *app, const QString &sessionName)
     : QObject()
-    , App("Server", app, sessionName)
+    , App(app, sessionName)
     , m_wnd(nullptr)
     , m_shortcutActions()
     , m_ignoreKeysTimer()
 {
+    setCurrentThreadName("Server");
+
     const QString serverName = clipboardServerName();
     m_server = new Server(serverName, this);
 
@@ -108,11 +112,11 @@ ClipboardServer::ClipboardServer(QApplication *app, const QString &sessionName)
     connect( qApp, &QCoreApplication::aboutToQuit,
              this, &ClipboardServer::onAboutToQuit );
 
-    connect( qApp, SIGNAL(commitDataRequest(QSessionManager&)),
-             this, SLOT(onCommitData(QSessionManager&)) );
-
-    connect( qApp, SIGNAL(saveStateRequest(QSessionManager&)),
-             this, SLOT(onSaveState(QSessionManager&)) );
+    connect( qApp, &QGuiApplication::commitDataRequest, this, &ClipboardServer::onCommitData );
+    connect( qApp, &QGuiApplication::saveStateRequest, this, &ClipboardServer::onSaveState );
+#if QT_VERSION >= QT_VERSION_CHECK(5,6,0)
+    qApp->setFallbackSessionManagementEnabled(false);
+#endif
 
     connect( m_wnd, SIGNAL(requestExit()),
              this, SLOT(maybeQuit()) );
@@ -155,10 +159,13 @@ void ClipboardServer::stopMonitoring()
     COPYQ_LOG("Terminating monitor");
 
     for (auto it = m_clients.constBegin(); it != m_clients.constEnd(); ++it) {
-        const auto actionId = it.value().proxy->actionId();
+        const auto &clientData = it.value();
+        if (!clientData.isValid())
+            continue;
+
+        const auto actionId = clientData.proxy->actionId();
         if ( actionId == m_monitor->id() ) {
-            const auto client = it.key();
-            client->sendMessage(QByteArray(), CommandStop);
+            clientData.client->sendMessage(QByteArray(), CommandStop);
             break;
         }
     }
@@ -166,7 +173,7 @@ void ClipboardServer::stopMonitoring()
 
 void ClipboardServer::startMonitoring()
 {
-    if (m_monitor || wasClosed() || !m_wnd->isMonitoringEnabled())
+    if (m_monitor || m_preventMonitorStart || !m_wnd->isMonitoringEnabled())
         return;
 
     COPYQ_LOG("Starting monitor");
@@ -207,7 +214,9 @@ void ClipboardServer::onCommandsSaved()
     }
 #endif
 
-    if ( m_monitor && hasScriptCommand(commands) ) {
+    const auto scriptCommandsHash_ = scriptCommandsHash(commands);
+    if ( m_monitor && scriptCommandsHash_ != m_scriptCommandsHash ) {
+        m_scriptCommandsHash = scriptCommandsHash_;
         stopMonitoring();
         startMonitoring();
     }
@@ -217,9 +226,7 @@ void ClipboardServer::onAboutToQuit()
 {
     COPYQ_LOG("Closing server.");
 
-    // wasClosed() is true after App::exit() is called and
-    // it prevents monitor process restarting.
-    Q_ASSERT(wasClosed());
+    m_preventMonitorStart = true;
 
     terminateClients(10000);
     m_server->close(); // No new connections can be open.
@@ -232,11 +239,14 @@ void ClipboardServer::onCommitData(QSessionManager &sessionManager)
 {
     COPYQ_LOG("Got commit data request from session manager.");
 
+    m_preventMonitorStart = true;
     const bool cancel = sessionManager.allowsInteraction() && !askToQuit();
     sessionManager.release();
 
     if (cancel) {
         sessionManager.cancel();
+        m_preventMonitorStart = false;
+        startMonitoring();
     } else {
         m_wnd->saveTabs();
 
@@ -316,8 +326,9 @@ bool ClipboardServer::hasRunningCommands() const
 void ClipboardServer::terminateClients(int waitMs)
 {
     for (auto it = m_clients.constBegin(); it != m_clients.constEnd(); ++it) {
-        const auto client = it.key();
-        client->sendMessage(QByteArray(), CommandStop);
+        const auto &clientData = it.value();
+        if (clientData.isValid())
+            clientData.client->sendMessage(QByteArray(), CommandStop);
     }
 
     waitForClientsToFinish(waitMs);
@@ -334,7 +345,7 @@ void ClipboardServer::waitForClientsToFinish(int waitMs)
 void ClipboardServer::onClientNewConnection(const ClientSocketPtr &client)
 {
     auto proxy = new ScriptableProxy(m_wnd, client.get());
-    m_clients.insert( client.get(), ClientData(client, proxy) );
+    m_clients.insert( client->id(), ClientData(client, proxy) );
     connect( this, &ClipboardServer::closeClients,
              client.get(), &ClientSocket::close );
     connect( client.get(), &ClientSocket::messageReceived,
@@ -347,16 +358,16 @@ void ClipboardServer::onClientNewConnection(const ClientSocketPtr &client)
 }
 
 void ClipboardServer::onClientMessageReceived(
-        const QByteArray &message, int messageCode, ClientSocket *client)
+        const QByteArray &message, int messageCode, ClientSocketId clientId)
 {
-    Q_UNUSED(client);
     switch (messageCode) {
     case CommandFunctionCall: {
-        auto proxy = m_clients.value(client).proxy;
-        if (!proxy)
+        const auto &clientData = m_clients.value(clientId);
+        if (!clientData.isValid())
             return;
-        const auto result = proxy->callFunction(message);
-        client->sendMessage(result, CommandFunctionCallReturnValue);
+
+        const auto result = clientData.proxy->callFunction(message);
+        clientData.client->sendMessage(result, CommandFunctionCallReturnValue);
         break;
     }
     default:
@@ -365,15 +376,15 @@ void ClipboardServer::onClientMessageReceived(
     }
 }
 
-void ClipboardServer::onClientDisconnected(ClientSocket *client)
+void ClipboardServer::onClientDisconnected(ClientSocketId clientId)
 {
-    m_clients.remove(client);
+    m_clients.remove(clientId);
 }
 
-void ClipboardServer::onClientConnectionFailed(ClientSocket *client)
+void ClipboardServer::onClientConnectionFailed(ClientSocketId clientId)
 {
     log("Client connection failed", LogWarning);
-    m_clients.remove(client);
+    m_clients.remove(clientId);
 }
 
 void ClipboardServer::onMonitorFinished()
@@ -410,28 +421,22 @@ bool ClipboardServer::eventFilter(QObject *object, QEvent *ev)
 {
     const QEvent::Type type = ev->type();
 
-    if ( m_ignoreKeysTimer.isActive()
-         && (type == QEvent::KeyPress
-             || type == QEvent::Shortcut
-             || type == QEvent::ShortcutOverride) )
-    {
-        ev->accept();
-        return true;
-    }
-
     if ( type == QEvent::KeyPress
          || type == QEvent::Shortcut
          || type == QEvent::ShortcutOverride )
     {
-        m_wnd->updateShortcuts();
-    }
+        if ( m_ignoreKeysTimer.isActive() ) {
+            ev->accept();
+            return true;
+        }
 
-    // Close menu on Escape key and give focus back to search edit or browser.
-    if (type == QEvent::KeyPress) {
-        QKeyEvent *keyevent = static_cast<QKeyEvent *>(ev);
-        if (keyevent->key() == Qt::Key_Escape) {
+        m_wnd->updateShortcuts();
+
+        // Close menu on Escape key and give focus back to search edit or browser.
+        if (type == QEvent::KeyPress) {
+            QKeyEvent *keyevent = static_cast<QKeyEvent *>(ev);
             QMenu *menu = qobject_cast<QMenu*>(object);
-            if (menu != nullptr) {
+            if (menu && keyevent->key() == Qt::Key_Escape) {
                 menu->close();
                 if (m_wnd->browseMode())
                     m_wnd->enterBrowseMode();
