@@ -66,9 +66,9 @@ QImage getImageData(const QMimeData &data)
 {
     // NOTE: Application hangs if using mulitple sessions and
     //       calling QMimeData::hasImage() on X11 clipboard.
-    COPYQ_LOG("Fetching image data from clipboard");
+    COPYQ_LOG_VERBOSE("Fetching image data from clipboard");
     const QImage image = data.imageData().value<QImage>();
-    COPYQ_LOG( QString("Image is %1").arg(image.isNull() ? "invalid" : "valid") );
+    COPYQ_LOG_VERBOSE( QString("Image is %1").arg(image.isNull() ? "invalid" : "valid") );
     return image;
 }
 
@@ -162,22 +162,16 @@ QByteArray getUtf8Data(const QMimeData &data, const QString &format)
     return data.data(format);
 }
 
-bool hasNonInternalFormats(const QStringList &formats) {
-    for (auto it = formats.constBegin(); it != formats.constEnd(); ++it) {
-        if ( !it->startsWith(COPYQ_MIME_PREFIX) ) {
+bool findFormatsWithPrefix(bool hasPrefix, const QString &prefix, const QVariantMap &data)
+{
+    for (auto it = data.constBegin(); it != data.constEnd(); ++it) {
+        if ( it.key().startsWith(prefix) == hasPrefix )
             return true;
-        }
     }
 
     return false;
 }
 
-QByteArray thisClipboardOwnerData()
-{
-    return qgetenv("COPYQ_SESSION_NAME")
-            + " " + QByteArray::number(QCoreApplication::applicationPid());
-}
-
 } // namespace
 
 bool isMainThread()
@@ -214,13 +208,23 @@ QVariantMap cloneData(const QMimeData &data, QStringList formats)
     QImage image;
     bool imageLoaded = false;
 
-    // Ignore image data if text is available.
+    /*
+     Some apps provide images even when copying huge spreadsheet, this can
+     block those apps while generating and providing the data.
+
+     This code removes ignores any image data if text is available.
+
+     Images in SVG and other XML formats are expected to be relatively small
+     so these doesn't have to be ignored.
+     */
     if ( formats.contains(mimeText) && data.hasFormat(mimeText) ) {
         const QString mimeImagePrefix = "image/";
         const auto first = std::remove_if(
                     std::begin(formats), std::end(formats),
                     [&mimeImagePrefix](const QString &format) {
-                        return format.startsWith(mimeImagePrefix);
+                        return format.startsWith(mimeImagePrefix)
+                            && !format.contains("xml")
+                            && !format.contains("svg");
                     });
         formats.erase(first, std::end(formats));
     }
@@ -292,11 +296,8 @@ QMimeData* createMimeData(const QVariantMap &data)
     for ( const auto &format : copyFormats )
         newClipboardData->setData( format, data[format].toByteArray() );
 
-#ifdef HAS_TESTS
-    // Don't set clipboard owner if monitor is only used to set clipboard for tests.
-    if ( !qApp->property("CopyQ_testing").toBool() )
-#endif
-        newClipboardData->setData( mimeOwner, thisClipboardOwnerData() );
+    if ( !copyFormats.contains(mimeOwner) )
+        newClipboardData->setData( mimeOwner, makeClipboardOwnerData() );
 
     // Set image data.
     const QStringList formats =
@@ -309,7 +310,7 @@ QMimeData* createMimeData(const QVariantMap &data)
     return newClipboardData.release();
 }
 
-bool ownsClipboardData(const QVariantMap &data)
+bool anySessionOwnsClipboardData(const QVariantMap &data)
 {
     return data.contains(mimeOwner);
 }
@@ -323,7 +324,7 @@ QString elideText(const QString &text, const QFont &font, const QString &format,
     QStringList lines = text.split('\n');
 
     // Ignore empty lines at beginning.
-    const QRegExp reNonEmpty(".*\\S.*");
+    static const QRegExp reNonEmpty(".*\\S.*");
     const int firstLine = qMax(0, lines.indexOf(reNonEmpty));
     const int lastLine = qMax(0, lines.lastIndexOf(reNonEmpty, firstLine + maxLines - 1));
 
@@ -346,7 +347,7 @@ QString elideText(const QString &text, const QFont &font, const QString &format,
 
     // Find common indentation.
     int commonIndent = lines.value(0).size();
-    const QRegExp reNonSpace("\\S");
+    static const QRegExp reNonSpace("\\S");
     for (const auto &line : lines) {
         const int lineIndent = line.indexOf(reNonSpace);
         if (lineIndent != -1 && lineIndent < commonIndent) {
@@ -381,13 +382,11 @@ QString textLabelForData(const QVariantMap &data, const QFont &font, const QStri
 {
     QString label;
 
-    const QStringList formats = data.keys();
-
     const QString notes = data.value(mimeItemNotes).toString();
 
     if ( data.contains(mimeHidden) ) {
         label = QObject::tr("<HIDDEN>", "Label for hidden/secret clipboard content");
-    } else if ( formats.contains(mimeText) || formats.contains(mimeUriList) ) {
+    } else if ( data.contains(mimeText) || data.contains(mimeUriList) ) {
         const QString text = getTextData(data);
         const int n = text.count(QChar('\n')) + 1;
 
@@ -401,9 +400,9 @@ QString textLabelForData(const QVariantMap &data, const QFont &font, const QStri
 
         const QString textWithNotes = notes.isEmpty() ? text : notes + ": " + text;
         return elideText(textWithNotes, font, label, escapeAmpersands, maxWidthPixels, maxLines);
-    } else if ( formats.indexOf(QRegExp("^image/.*")) != -1 ) {
+    } else if ( findFormatsWithPrefix(true, "image/", data) ) {
         label = QObject::tr("<IMAGE>", "Label for image in clipboard");
-    } else if ( !hasNonInternalFormats(formats) ) {
+    } else if ( findFormatsWithPrefix(false, COPYQ_MIME_PREFIX, data) ) {
         label = QObject::tr("<EMPTY>", "Label for empty clipboard");
     } else {
         label = QObject::tr("<DATA>", "Label for data in clipboard");
@@ -453,24 +452,6 @@ QString dataToText(const QByteArray &bytes, const QString &mime)
     return codec->toUnicode(bytes);
 }
 
-bool clipboardContains(ClipboardMode mode, const QVariantMap &data)
-{
-    const QMimeData *clipboardData = ::clipboardData(mode);
-    if (!clipboardData)
-        return false;
-
-    for (auto it = data.constBegin(); it != data.constEnd(); ++it) {
-        const auto &format = it.key();
-        if ( !format.startsWith(COPYQ_MIME_PREFIX)
-             && it.value().toByteArray() != getUtf8Data(*clipboardData, format) )
-        {
-            return false;
-        }
-    }
-
-    return true;
-}
-
 bool isClipboardData(const QVariantMap &data)
 {
     return data.value(mimeClipboardMode).toByteArray().isEmpty();
@@ -558,9 +539,12 @@ void acceptDrag(QDropEvent *event)
     }
 }
 
-bool ownsClipboardData(ClipboardMode mode)
+QByteArray makeClipboardOwnerData()
 {
-    return clipboardOwnerData(mode) == thisClipboardOwnerData();
+    static int id = 0;
+    return qgetenv("COPYQ_SESSION_NAME")
+           + " " + currentThreadLabel()
+           + "/" + QByteArray::number(++id);
 }
 
 QByteArray clipboardOwnerData(ClipboardMode mode)
