@@ -52,7 +52,7 @@ bool isMainThread();
 
 const QMimeData *clipboardData(ClipboardMode mode = ClipboardMode::Clipboard);
 
-bool ownsClipboardData(ClipboardMode mode);
+QByteArray makeClipboardOwnerData();
 
 QByteArray clipboardOwnerData(ClipboardMode mode);
 
@@ -65,7 +65,7 @@ QVariantMap cloneData(const QMimeData &data);
 QMimeData* createMimeData(const QVariantMap &data);
 
 /** Return true if clipboard content was created by any session of this application. */
-bool ownsClipboardData(const QVariantMap &data);
+bool anySessionOwnsClipboardData(const QVariantMap &data);
 
 /**
  * Elide text with given font or to maximum length
@@ -108,8 +108,6 @@ void initSingleShotTimer(QTimer *timer, int milliseconds, const QObject *object
 
 QString dataToText(const QByteArray &bytes, const QString &mime);
 
-bool clipboardContains(ClipboardMode mode, const QVariantMap &data);
-
 bool isClipboardData(const QVariantMap &data);
 
 /**
