@@ -250,6 +250,8 @@ class ClipboardBrowser : public QListView
 
         QVariantMap itemData(const QModelIndex &index) const;
 
+        bool isLoaded() const;
+
     public slots:
         /**
          * Save items to configuration.
@@ -368,8 +370,6 @@ class ClipboardBrowser : public QListView
         void updateCurrent();
 
     private:
-        bool isLoaded() const;
-
         /**
          * Hide row if filtered out, otherwise show.
          * @return true only if hidden
@@ -424,6 +424,8 @@ class ClipboardBrowser : public QListView
 
         QModelIndex firstUnpinnedIndex() const;
 
+        void dragDropScroll();
+
         ItemSaverPtr m_itemSaver;
         QString m_tabName;
         ClipboardModel m;
@@ -433,6 +435,7 @@ class ClipboardBrowser : public QListView
         QTimer m_timerUpdateSizes;
         QTimer m_timerUpdateItemWidgets;
         QTimer m_timerUpdateCurrent;
+        QTimer m_timerDragDropScroll;
         bool m_ignoreMouseMoveWithButtonPressed = false;
         bool m_resizing = false;
 
