@@ -53,17 +53,9 @@ const int lightThreshold = 100;
 
 QPointer<QObject> activePaintDevice;
 
-void replaceColor(QPixmap *pix, const QColor &from, const QColor &to)
+bool hasNormalIcon()
 {
-    if (from == to)
-        return;
-
-    QPixmap pix2( pix->size() );
-    pix2.fill(to);
-    pix2.setMask( pix->createMaskFromColor(from, Qt::MaskOutColor) );
-
-    QPainter p(pix);
-    p.drawPixmap(0, 0, pix2);
+    return QIcon::hasThemeIcon(COPYQ_ICON_NAME "-normal");
 }
 
 QString sessionName()
@@ -82,22 +74,10 @@ QColor colorFromEnv(const char *envVaribleName)
     return QColor( textFromEnv(envVaribleName) );
 }
 
-QColor appIconColorHelper()
-{
-    const auto color = colorFromEnv("COPYQ_APP_COLOR");
-    return color.isValid() ? QColor(color) : QColor(0x7f, 0xca, 0x9b);
-}
-
-QColor appIconColor()
-{
-    static const QColor color = appIconColorHelper();
-    return color;
-}
-
 QColor sessionNameToColor(const QString &name)
 {
     if (name.isEmpty())
-        return appIconColor();
+        return QColor();
 
     int r = 0;
     int g = 0;
@@ -124,7 +104,7 @@ QColor sessionNameToColor(const QString &name)
 QColor sessionIconColorHelper()
 {
     const auto color = colorFromEnv("COPYQ_SESSION_COLOR");
-    return color.isValid() ? QColor(color) : sessionNameToColor( sessionName() );
+    return color.isValid() ? color : sessionNameToColor( sessionName() );
 }
 
 QColor &sessionIconColorVariable()
@@ -171,16 +151,53 @@ QPixmap pixmapFromFile(const QString &path, QSize size)
     return pix;
 }
 
-QString imagePathFromPrefix(const QString &iconSuffix, const QString &resources)
+QString iconPath(const QString &iconSuffix)
 {
 #ifdef COPYQ_ICON_PREFIX
-    const QString fileName(COPYQ_ICON_PREFIX + iconSuffix);
+    const QString fileName(COPYQ_ICON_PREFIX + iconSuffix + ".svg");
     if ( QFile::exists(fileName) )
         return fileName;
 #else
     Q_UNUSED(iconSuffix)
 #endif
-    return imagesRecourcePath + resources;
+    return imagesRecourcePath + QLatin1String("icon") + iconSuffix;
+}
+
+QPixmap appPixmap(const QString &iconSuffix, QSize size)
+{
+    if ( iconSuffix.isEmpty() && hasNormalIcon() )
+        return appPixmap("-normal", size);
+
+    const auto icon = QIcon::fromTheme(COPYQ_ICON_NAME + iconSuffix);
+
+    QPixmap pix;
+
+    if ( icon.isNull() ) {
+        const auto path = iconPath(iconSuffix);
+        pix = pixmapFromFile(path, size);
+    } else {
+        pix = icon.pixmap(size)
+                .scaled(size, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+    }
+
+    pix.setDevicePixelRatio(1);
+
+    return pix;
+}
+
+void replaceColor(QPixmap *pix, const QString &iconSuffix, const QColor &targetColor)
+{
+    auto pix2 = appPixmap(iconSuffix + "-mask", pix->size());
+
+    {
+        QPainter p1(&pix2);
+        p1.setCompositionMode(QPainter::CompositionMode_SourceAtop);
+        p1.fillRect(pix->rect(), targetColor);
+    }
+
+    QPainter p(pix);
+    p.setCompositionMode(QPainter::CompositionMode_SourceAtop);
+    p.drawPixmap(0, 0, pix2);
 }
 
 void drawFontIcon(QPixmap *pix, ushort id, int w, int h, const QColor &color)
@@ -338,11 +355,42 @@ private:
     QColor m_tagColor;
 };
 
-class ImageIconEngine : public BaseIconEngine
+class FontIconEngine : public BaseIconEngine
 {
 public:
-    ImageIconEngine(const QString &iconName, const QString &tag, QColor tagColor)
+    FontIconEngine(ushort iconId, const QString &tag, QColor tagColor)
         : BaseIconEngine(tag, tagColor)
+        , m_iconId(iconId)
+    {
+    }
+
+    QIconEngine *clone() const override
+    {
+        return new FontIconEngine(*this);
+    }
+
+    QPixmap doCreatePixmap(QSize size, QIcon::Mode mode, QIcon::State, QPainter *painter) override
+    {
+        QPixmap pixmap(size);
+        pixmap.fill(Qt::transparent);
+
+        if (m_iconId == 0)
+            return pixmap;
+
+        drawFontIcon( &pixmap, m_iconId, size.width(), size.height(), colorForMode(painter, mode) );
+
+        return pixmap;
+    }
+
+private:
+    ushort m_iconId;
+};
+
+class ImageIconEngine : public FontIconEngine
+{
+public:
+    ImageIconEngine(const QString &iconName, ushort fallbackIconId, const QString &tag, QColor tagColor)
+        : FontIconEngine(fallbackIconId, tag, tagColor)
         , m_iconName(iconName)
     {
     }
@@ -355,7 +403,7 @@ public:
     QPixmap doCreatePixmap(QSize size, QIcon::Mode mode, QIcon::State state, QPainter *painter) override
     {
         if ( m_iconName.isEmpty() )
-            return QPixmap();
+            return FontIconEngine::doCreatePixmap(size, mode, state, painter);
 
         // Tint tab icons.
         if ( m_iconName.startsWith(imagesRecourcePath + QString("tab_")) ) {
@@ -402,44 +450,13 @@ public:
             return pixmap;
         }
 
-        return QPixmap();
+        return FontIconEngine::doCreatePixmap(size, mode, state, painter);
     }
 
 private:
     QString m_iconName;
 };
 
-class FontIconEngine : public BaseIconEngine
-{
-public:
-    FontIconEngine(ushort iconId, const QString &tag, QColor tagColor)
-        : BaseIconEngine(tag, tagColor)
-        , m_iconId(iconId)
-    {
-    }
-
-    QIconEngine *clone() const override
-    {
-        return new FontIconEngine(*this);
-    }
-
-    QPixmap doCreatePixmap(QSize size, QIcon::Mode mode, QIcon::State, QPainter *painter) override
-    {
-        QPixmap pixmap(size);
-        pixmap.fill(Qt::transparent);
-
-        if (m_iconId == 0)
-            return pixmap;
-
-        drawFontIcon( &pixmap, m_iconId, size.width(), size.height(), colorForMode(painter, mode) );
-
-        return pixmap;
-    }
-
-private:
-    ushort m_iconId;
-};
-
 class AppIconEngine : public BaseIconEngine
 {
 public:
@@ -457,45 +474,15 @@ public:
     QPixmap doCreatePixmap(QSize size, QIcon::Mode, QIcon::State, QPainter *) override
     {
         const bool running = m_iconType == AppIconRunning;
-        const QString suffix = running ? "-busy" : "-normal";
-        const QString sessionName = ::sessionName();
+        const auto suffix = running ? QLatin1String("-busy") : QLatin1String("");
 
-        QIcon icon;
+        auto pix = appPixmap(suffix, size);
 
-        if (sessionName.isEmpty())
-            icon = QIcon::fromTheme(COPYQ_ICON_NAME + suffix);
-        else
-            icon = QIcon::fromTheme(COPYQ_ICON_NAME "_" + sessionName + "-" + suffix);
-
-        QPixmap pix;
-
-        const auto sessionColor = sessionIconColor();
-        const auto appColor = appIconColor();
-        const bool colorize = (sessionColor != appColor);
-
-        // Colorize icon in higher resolution.
-        const auto colorizeSize = 256;
-        const auto sizeFactor = (colorize && size.height() < colorizeSize)
-                ? (colorizeSize / size.height())
-                : 1;
-        size *= sizeFactor;
-
-        if ( icon.isNull() ) {
-            const auto path = imagePathFromPrefix(suffix + ".svg", running ? "icon-running" : "icon");
-            pix = pixmapFromFile(path, size);
-        } else {
-            pix = icon.pixmap(size)
-                    .scaled(size, Qt::KeepAspectRatio, Qt::SmoothTransformation);
-        }
-
-        pix.setDevicePixelRatio(1);
-
-        if (colorize)
-            replaceColor(&pix, appColor, sessionColor);
-
-        if (sizeFactor != 1) {
-            size /= sizeFactor;
-            pix = pix.scaled(size, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+        // If copyq-normal icon exist in theme, omit changing color.
+        if ( !hasNormalIcon() ) {
+            const auto sessionColor = sessionIconColor();
+            if ( sessionColor.isValid() )
+                replaceColor(&pix, suffix, sessionColor);
         }
 
         return pix;
@@ -521,7 +508,7 @@ public:
     {
         if ( canUseFontIcon(iconId, iconName) )
             return QIcon( new FontIconEngine(iconId, tag, tagColor) );
-        return QIcon( new ImageIconEngine(iconName, tag, tagColor) );
+        return QIcon( new ImageIconEngine(iconName, iconId, tag, tagColor) );
     }
 
     static QIcon createIcon(AppIconType iconType)
@@ -626,7 +613,7 @@ unsigned short toIconId(const QString &fileNameOrId)
 
 void setSessionIconColor(QColor color)
 {
-    sessionIconColorVariable() = color;
+    sessionIconColorVariable() = color.isValid() ? color : sessionIconColorHelper();
 }
 
 void setSessionIconTag(const QString &tag)
