@@ -197,7 +197,7 @@ class ThreadNameDecorator : public Decorator
 {
 public:
     explicit ThreadNameDecorator(const QFont &font, QObject *parent)
-        : Decorator(QRegExp(" [A-Z][a-z]+-[0-9-]+:"), parent)
+        : Decorator(QRegExp("<[A-Za-z]+-[0-9-]+>"), parent)
     {
         QFont boldFont = font;
         boldFont.setBold(true);
@@ -215,8 +215,10 @@ private:
         m_format.setForeground( QColor::fromHsv(h, 150, 100) );
 
         const auto bg =
-                text.startsWith(" Server-") ? QColor::fromHsv(60, 80, 255)
-              : text.startsWith(" Monitor-") ? QColor::fromHsv(200, 80, 255)
+                text.startsWith("<Server-") ? QColor::fromRgb(255, 255, 200)
+              : text.startsWith("<monitor") ? QColor::fromRgb(220, 240, 255)
+              : text.startsWith("<provide") ? QColor::fromRgb(220, 255, 220)
+              : text.startsWith("<synchronize") ? QColor::fromRgb(220, 255, 240)
               : QColor(Qt::white);
         m_format.setBackground(bg);
 
