@@ -20,6 +20,7 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
+#include "app/clipboardmanager.h"
 #include "common/clipboardmode.h"
 #include "common/command.h"
 #include "gui/clipboardbrowsershared.h"
@@ -358,8 +359,8 @@ public slots:
     bool isInternalActionId(int id) const;
 
     void setClipboard(const QVariantMap &data, ClipboardMode mode);
-    void setClipboard(const QVariantMap &data);
-    void setClipboardAndWait(const QVariantMap &data, ClipboardMode mode);
+    void setClipboardAndSelection(const QVariantMap &data);
+    void moveToClipboard(ClipboardBrowser *c, int row);
 
     /** Show/hide main window. Return true only if window is shown. */
     bool toggleVisible();
@@ -635,8 +636,6 @@ private:
     void updateActionShortcuts(int id);
     void updateActionShortcuts();
 
-    void pasteClipboard(const PlatformWindowPtr &window);
-
     QAction *actionForMenuItem(int id, QWidget *parent, Qt::ShortcutContext context);
 
     void addMenuItems(TrayMenu *menu, ClipboardBrowser *c, int maxItemCount, const QString &searchText);
@@ -652,6 +651,8 @@ private:
 
     Action *runScript(const QString &script, const QVariantMap &data = QVariantMap());
 
+    void activateCurrentItemHelper();
+
     ConfigurationManager *cm;
     Ui::MainWindow *ui;
 
@@ -703,6 +704,8 @@ private:
 
     bool m_showItemPreview;
 
+    bool m_activatingItem = false;
+
     QList<QKeySequence> m_disabledShortcuts;
 
     QVector< QPointer<QAction> > m_actions;
@@ -715,6 +718,8 @@ private:
     MenuMatchCommands m_trayMenuMatchCommands;
     MenuMatchCommands m_itemMenuMatchCommands;
 
+    ClipboardManager m_clipboardManager;
+
 #ifdef HAS_TESTS
     /// Key clicks sequence number last returned by sendKeyClicks().
     uint m_sentKeyClicks = 0;
