@@ -323,19 +323,15 @@ ItemWidget *ItemDelegate::updateCache(const QModelIndex &index, const QVariantMa
     return w;
 }
 
-void ItemDelegate::invalidateCache()
-{
-    for(auto &w : m_cache)
-        w.reset();
-}
-
 bool ItemDelegate::invalidateHidden(QWidget *widget)
 {
     if ( widget->isVisible() && m_view->isVisible() )
         return false;
 
     const auto row = findWidgetRow(widget);
-    Q_ASSERT(row != -1);
+    if (row == -1)
+        return false;
+
     const auto index = m_view->index(row);
     setIndexWidget(index, nullptr);
     return true;
