@@ -21,9 +21,11 @@
 
 #include "app/applicationexceptionhandler.h"
 #include "common/log.h"
+#include "copyqpasteboardmime.h"
 #include "foregroundbackgroundfilter.h"
 #include "macplatformwindow.h"
 #include "platform/mac/macactivity.h"
+#include "urlpasteboardmime.h"
 #include "macclipboard.h"
 
 #include <QApplication>
@@ -37,6 +39,23 @@
 #include <Carbon/Carbon.h>
 
 namespace {
+    class ClipboardApplication : public QApplication
+    {
+    public:
+        ClipboardApplication(int &argc, char **argv)
+            : QApplication(argc, argv)
+            , m_pasteboardMime()
+            , m_pasteboardMimeUrl(QLatin1String("public.url"))
+            , m_pasteboardMimeFileUrl(QLatin1String("public.file-url"))
+        {
+        }
+
+    private:
+        CopyQPasteboardMime m_pasteboardMime;
+        UrlPasteboardMime m_pasteboardMimeUrl;
+        UrlPasteboardMime m_pasteboardMimeFileUrl;
+    };
+
     template<typename T> inline T* objc_cast(id from)
     {
         if ([from isKindOfClass:[T class]]) {
@@ -158,7 +177,7 @@ QCoreApplication *MacPlatform::createConsoleApplication(int &argc, char **argv)
 
 QApplication *MacPlatform::createServerApplication(int &argc, char **argv)
 {
-    QApplication *app = new Activity<QApplication>(argc, argv, MacActivity::Background, "CopyQ Server");
+    QApplication *app = new Activity<ClipboardApplication>(argc, argv, MacActivity::Background, "CopyQ Server");
 
     // Switch the app to foreground when in foreground
     ForegroundBackgroundFilter::installFilter(app);
@@ -166,14 +185,14 @@ QApplication *MacPlatform::createServerApplication(int &argc, char **argv)
     return app;
 }
 
-QApplication *MacPlatform::createMonitorApplication(int &argc, char **argv)
+QGuiApplication *MacPlatform::createMonitorApplication(int &argc, char **argv)
 {
-    return new Activity<QApplication>(argc, argv, MacActivity::Background, "CopyQ clipboard monitor");
+    return new Activity<ClipboardApplication>(argc, argv, MacActivity::Background, "CopyQ clipboard monitor");
 }
 
-QApplication *MacPlatform::createClipboardProviderApplication(int &argc, char **argv)
+QGuiApplication *MacPlatform::createClipboardProviderApplication(int &argc, char **argv)
 {
-    return new Activity<QApplication>(argc, argv, MacActivity::Background, "CopyQ clipboard provider");
+    return new Activity<ClipboardApplication>(argc, argv, MacActivity::Background, "CopyQ clipboard provider");
 }
 
 QCoreApplication *MacPlatform::createClientApplication(int &argc, char **argv)
