@@ -44,8 +44,6 @@
 #include <QClipboard>
 #include <QDateTime>
 #include <QDir>
-#include <QDesktopServices>
-#include <QElapsedTimer>
 #include <QFile>
 #include <QFileInfo>
 #include <QMap>
@@ -63,8 +61,6 @@
 #include <QTextCodec>
 #include <QTimer>
 
-#include <cmath>
-
 Q_DECLARE_METATYPE(QByteArray*)
 Q_DECLARE_METATYPE(QFile*)
 
@@ -73,8 +69,6 @@ namespace {
 const char *const programName = "CopyQ Clipboard Manager";
 const char *const mimeIgnore = COPYQ_MIME_PREFIX "ignore";
 
-const int setClipboardMaxRetries = 3;
-
 QString helpHead()
 {
     return Scriptable::tr("Usage: copyq [%1]").arg(Scriptable::tr("COMMAND")) + "\n\n"
@@ -96,13 +90,6 @@ QString argumentError()
     return Scriptable::tr("Invalid number of arguments!");
 }
 
-bool clipboardContains(
-        ClipboardMode mode, ScriptableProxy *proxy,
-        const QString &format, const QByteArray &content)
-{
-    return content == proxy->getClipboardData(format, mode);
-}
-
 QByteArray *getByteArray(const QScriptValue &value, const Scriptable *scriptable)
 {
     if (value.scriptClass() == scriptable->byteArrayClass())
@@ -402,7 +389,7 @@ QByteArray readReply(QNetworkReply *reply, const Scriptable &scriptable)
 {
     QByteArray data;
     while ( !reply->isFinished() ) {
-        if ( !scriptable.isConnected() )
+        if ( !scriptable.canContinue() )
             return QByteArray();
         waitFor(100);
         if ( reply->waitForReadyRead(100) )
@@ -488,9 +475,7 @@ bool isInternalDataFormat(const QString &format)
         || format == mimeSelectedItems
         || format == mimeCurrentItem
         || format == mimeShortcut
-        || format == mimeOutputTab
-        || format == mimeSyncToClipboard
-        || format == mimeSyncToSelection;
+        || format == mimeOutputTab;
 }
 
 QVariantMap copyWithoutInternalData(const QVariantMap &data) {
@@ -515,61 +500,6 @@ QStringList monitorFormatsToSave()
     return factory.formatsToSave();
 }
 
-#ifdef HAS_MOUSE_SELECTIONS
-class SynchronizeSelectionTimer : public QObject
-{
-public:
-    explicit SynchronizeSelectionTimer(ScriptableProxy *scriptableProxy)
-        : QObject()
-        , m_scriptableProxy(scriptableProxy)
-    {
-        m_timer.setSingleShot(true);
-        QObject::connect(
-            &m_timer, &QTimer::timeout,
-            this, &SynchronizeSelectionTimer::synchronize);
-    }
-
-    void start(ClipboardMode targetClipboardMode, const QVariantMap &data)
-    {
-        m_targetClipboardMode = targetClipboardMode;
-        m_data = data;
-        if ( !m_timer.isActive() )
-            m_timer.start(0);
-        syncLog("Started");
-    }
-
-    void reset()
-    {
-        m_data.clear();
-        m_timer.start(500);
-        syncLog("Reset");
-    }
-
-private:
-    void synchronize()
-    {
-        if ( m_data.isEmpty() )
-            return;
-
-        syncLog("Synchronizing");
-        m_scriptableProxy->setClipboard(m_data, m_targetClipboardMode);
-        syncLog("Synchronized");
-    }
-
-    void syncLog(const char *message) const
-    {
-        COPYQ_LOG( QString("Sync to %1: %2")
-                   .arg(m_targetClipboardMode == ClipboardMode::Clipboard ? "clipboard" : "selection")
-                   .arg(message) );
-    }
-
-    ScriptableProxy *m_scriptableProxy;
-    ClipboardMode m_targetClipboardMode = ClipboardMode::Clipboard;
-    QVariantMap m_data;
-    QTimer m_timer;
-};
-#endif // HAS_MOUSE_SELECTIONS
-
 } // namespace
 
 Scriptable::Scriptable(
@@ -586,7 +516,6 @@ Scriptable::Scriptable(
     , m_temporaryFileClass(nullptr)
     , m_inputSeparator("\n")
     , m_input()
-    , m_connected(true)
 {
     const QScriptEngine::QObjectWrapOptions opts =
               QScriptEngine::ExcludeChildObjects
@@ -765,11 +694,6 @@ void Scriptable::throwImportError(const QString &filePath)
     throwError( tr("Cannot import file \"%1\"!").arg(filePath) );
 }
 
-void Scriptable::sendMessageToClient(const QByteArray &message, int exitCode)
-{
-    emit sendMessage(message, exitCode);
-}
-
 QScriptValue Scriptable::getGlobal()
 {
     return m_engine->globalObject();
@@ -945,7 +869,7 @@ void Scriptable::exit()
 {
     m_skipArguments = 0;
     QByteArray message = fromString( tr("Terminating server.\n") );
-    sendMessageToClient(message, CommandPrint);
+    print(message);
     m_proxy->exit();
 }
 
@@ -1112,8 +1036,10 @@ void Scriptable::select()
     m_skipArguments = 1;
     QScriptValue value = argument(0);
     int row;
-    if ( toInt(value, &row) )
-        m_proxy->browserMoveToClipboard(row);
+    if ( !toInt(value, &row) )
+        throwError(argumentError());
+
+    m_proxy->browserMoveToClipboard(row);
 }
 
 void Scriptable::next()
@@ -1575,7 +1501,7 @@ QScriptValue Scriptable::input()
 
     if ( !getByteArray(m_input, this) ) {
         emit readInput();
-        if (m_connected) {
+        if (canContinue()) {
             QEventLoop loop;
             connect(this, &Scriptable::finished, &loop, &QEventLoop::quit);
             connect(this, &Scriptable::dataReceived, &loop, &QEventLoop::quit);
@@ -1667,25 +1593,20 @@ QScriptValue Scriptable::removeData()
 void Scriptable::print(const QScriptValue &value)
 {
     m_skipArguments = 1;
-    sendMessageToClient(makeByteArray(value), CommandPrint);
+    print(makeByteArray(value));
 }
 
 void Scriptable::abort()
 {
     m_skipArguments = 0;
-    m_connected = false;
-
-    QScriptEngine *eng = engine() ? engine() : m_engine;
-    if (eng)
-        eng->abortEvaluation();
-
-    emit finished();
+    abortEvaluation(m_action ? Abort::CurrentEvaluation : Abort::AllEvaluations);
 }
 
 void Scriptable::fail()
 {
     m_skipArguments = 0;
-    sendMessageToClient("", CommandError);
+    m_failed = true;
+    abortEvaluation(Abort::CurrentEvaluation);
 }
 
 #ifdef HAS_TESTS
@@ -1716,7 +1637,7 @@ void Scriptable::keys()
         // Make sure all keys are send (shortcuts are postponed because they can be blocked by modal windows).
         while ( !m_proxy->keysSent() ) {
             QCoreApplication::processEvents();
-            if (!m_connected) {
+            if (!canContinue()) {
                 throwError("Disconnected");
                 return;
             }
@@ -1925,7 +1846,7 @@ QScriptValue Scriptable::execute()
 
     QScriptValue actionResult = m_engine->newObject();
     actionResult.setProperty( "stdout", newByteArray(m_executeStdoutData) );
-    actionResult.setProperty( "stderr", action.errorOutput() );
+    actionResult.setProperty( "stderr", getTextData(action.errorOutput()) );
     actionResult.setProperty( "exit_code", action.exitCode() );
 
     m_executeStdoutData.clear();
@@ -2080,7 +2001,7 @@ void Scriptable::sleep()
         return;
     }
 
-    if (m_connected) {
+    if (canContinue()) {
         QEventLoop loop;
         connect(this, &Scriptable::finished, &loop, &QEventLoop::quit);
 
@@ -2243,7 +2164,6 @@ void Scriptable::onClipboardChanged()
     if (!hasData()) {
         updateClipboardData();
     } else if (runAutomaticCommands()) {
-        synchronizeSelection();
         saveData();
         updateClipboardData();
     } else {
@@ -2262,6 +2182,16 @@ void Scriptable::onHiddenClipboardChanged()
     eval("updateClipboardData()");
 }
 
+void Scriptable::synchronizeToSelection()
+{
+    synchronizeSelection(ClipboardMode::Selection);
+}
+
+void Scriptable::synchronizeFromSelection()
+{
+    synchronizeSelection(ClipboardMode::Clipboard);
+}
+
 void Scriptable::setClipboardData()
 {
     auto data = copyWithoutInternalData(m_data);
@@ -2280,19 +2210,6 @@ void Scriptable::setTitle()
     m_proxy->setTitle(title);
 }
 
-void Scriptable::synchronizeSelection()
-{
-#ifdef HAS_MOUSE_SELECTIONS
-    const bool syncToSelection = m_data.contains(mimeSyncToSelection);
-    const bool syncToClipboard = m_data.contains(mimeSyncToClipboard);
-    if (!syncToClipboard && !syncToSelection)
-        return;
-
-    const auto targetClipboardMode = syncToSelection ? ClipboardMode::Selection : ClipboardMode::Clipboard;
-    emit startSynchronizeSelectionTimer(targetClipboardMode, m_data);
-#endif
-}
-
 void Scriptable::saveData()
 {
     const QString outputTab = getTextData(m_data, mimeOutputTab);
@@ -2381,23 +2298,13 @@ void Scriptable::monitorClipboard()
 
     ClipboardMonitor monitor( monitorFormatsToSave() );
 
-#ifdef HAS_MOUSE_SELECTIONS
-    connect( QApplication::clipboard(), &QClipboard::changed,
-             this, [this]() {
-                m_data.remove(mimeSyncToSelection);
-                m_data.remove(mimeSyncToClipboard);
-                emit resetSynchronizeSelectionTimer();
-            });
-    SynchronizeSelectionTimer timer(m_proxy);
-    connect( this, &Scriptable::resetSynchronizeSelectionTimer, &timer, &SynchronizeSelectionTimer::reset );
-    connect( this, &Scriptable::startSynchronizeSelectionTimer, &timer, &SynchronizeSelectionTimer::start );
-#endif
-
     QEventLoop loop;
     connect(this, &Scriptable::finished, &loop, &QEventLoop::quit);
     connect(this, &Scriptable::stop, &loop, &QEventLoop::quit);
-    connect( &monitor, &ClipboardMonitor::runScriptRequest,
-             this, &Scriptable::onMonitorRunScriptRequest );
+    connect( &monitor, &ClipboardMonitor::clipboardChanged,
+             this, &Scriptable::onMonitorClipboardChanged );
+    connect( &monitor, &ClipboardMonitor::synchronizeSelection,
+             this, &Scriptable::onSynchronizeSelection );
     loop.exec();
 }
 
@@ -2426,29 +2333,42 @@ void Scriptable::onExecuteOutput(const QByteArray &output)
     }
 }
 
-void Scriptable::onMonitorRunScriptRequest(const QString &script, const QVariantMap &data)
+void Scriptable::onMonitorClipboardChanged(const QVariantMap &data, ClipboardOwnership ownership)
 {
-    COPYQ_LOG("Monitor eval: " + script);
+    COPYQ_LOG( QString("onMonitorClipboardChanged: %1 %2, owner is \"%3\"")
+               .arg(ownership == ClipboardOwnership::Own ? "own"
+                  : ownership == ClipboardOwnership::Foreign ? "foreign"
+                  : "hidden")
+               .arg(isClipboardData(data) ? "clipboard" : "selection")
+               .arg(getTextData(data, mimeOwner)) );
+
     m_data = data;
-    m_proxy->setActionData(m_actionId, m_data);
-    eval(script);
-    if ( engine()->hasUncaughtException() ) {
-        const auto exceptionText = processUncaughtException("ClipboardMonitor::" + script);
-        const auto response = createScriptErrorMessage(exceptionText).toUtf8();
-        sendMessageToClient(response, CommandException);
-    }
+    setActionData();
+
+    if (ownership == ClipboardOwnership::Own)
+        onOwnClipboardChanged();
+    else if (ownership == ClipboardOwnership::Hidden)
+        onHiddenClipboardChanged();
+    else
+        onClipboardChanged();
+
+    processUncaughtMonitorException("ClipboardMonitor::onMonitorClipboardChanged");
 }
 
-void Scriptable::onProvidedClipboardChanged()
+void Scriptable::onSynchronizeSelection(ClipboardMode sourceMode, const QString &text, uint targetTextHash)
 {
-    if ( !ownsClipboardData(ClipboardMode::Clipboard) )
-        emit finished();
-}
-
-void Scriptable::onProvidedSelectionChanged()
-{
-    if ( !ownsClipboardData(ClipboardMode::Selection) )
-        emit finished();
+#ifdef HAS_MOUSE_SELECTIONS
+    auto data = createDataMap(mimeText, text);
+    data[COPYQ_MIME_PREFIX "target-text-hash"] = QByteArray::number(targetTextHash);
+    const auto command = sourceMode == ClipboardMode::Clipboard
+        ? "copyq synchronizeToSelection"
+        : "copyq synchronizeFromSelection";
+    m_proxy->runInternalAction(data, command);
+#else
+    Q_UNUSED(text);
+    Q_UNUSED(sourceMode);
+    Q_UNUSED(targetTextHash);
+#endif
 }
 
 bool Scriptable::sourceScriptCommands()
@@ -2459,9 +2379,9 @@ bool Scriptable::sourceScriptCommands()
         eval(command.cmd, command.name);
         engine()->popContext();
         if ( engine()->hasUncaughtException() ) {
-            const auto exceptionText = processUncaughtException("ScriptCommand::" + command.cmd);
-            const auto response = createScriptErrorMessage(exceptionText).toUtf8();
-            sendMessageToClient(response, CommandException);
+            const auto exceptionText = processUncaughtException("ScriptCommand::" + command.name);
+            const auto message = createScriptErrorMessage(exceptionText).toUtf8();
+            printError(message);
             return false;
         }
     }
@@ -2469,84 +2389,84 @@ bool Scriptable::sourceScriptCommands()
     return true;
 }
 
-void Scriptable::executeArguments(const QStringList &args)
+int Scriptable::executeArguments(const QStringList &args)
 {
-    bool hasData;
-    m_actionId = qgetenv("COPYQ_ACTION_ID").toInt(&hasData);
-    const auto actionData = hasData ? m_proxy->getActionData(m_actionId) : QVariantMap();
-    m_data = actionData;
-
-    QByteArray response;
-    int exitCode;
-
     if ( args.isEmpty() ) {
         logScriptError("Bad command syntax");
-        exitCode = CommandBadSyntax;
-    } else {
-        /* Special arguments:
-             * "-"  read this argument from stdin
-             * "--" read all following arguments without control sequences
-             */
-        QScriptValueList fnArgs;
-        bool readRaw = false;
-        for (const auto &arg : args) {
-            if (readRaw) {
-                fnArgs.append( newByteArray(arg.toUtf8()) );
-            } else if (arg == "--") {
-                readRaw = true;
-            } else if (arg == "-") {
-                fnArgs.append( input() );
-            } else if (arg == "-e") {
-                fnArgs.append("eval");
-            } else {
-                const auto unescapedArg = parseCommandLineArgument(arg);
-                const auto value = newByteArray( unescapedArg.toUtf8() );
-                fnArgs.append(value);
-            }
-        }
+        return CommandBadSyntax;
+    }
 
-        if ( !sourceScriptCommands() )
-            return;
+    if ( !sourceScriptCommands() )
+        return CommandError;
 
-        QString cmd;
-        QScriptValue result;
-
-        int skipArguments = 0;
-        while ( skipArguments < fnArgs.size() && !m_engine->hasUncaughtException() ) {
-            if ( result.isFunction() ) {
-                m_skipArguments = -1;
-                result = result.call( QScriptValue(), fnArgs.mid(skipArguments) );
-                if (m_skipArguments == -1)
-                    break;
-                skipArguments += m_skipArguments;
-            } else {
-                cmd = toString(fnArgs[skipArguments], this);
-                result = eval(cmd);
-                ++skipArguments;
-            }
-        }
-
-        if ( result.isFunction() && !m_engine->hasUncaughtException() )
-            result = result.call( QScriptValue(), fnArgs.mid(skipArguments) );
-
-        if ( m_engine->hasUncaughtException() ) {
-            const auto exceptionText = processUncaughtException(cmd);
-            response = createScriptErrorMessage(exceptionText).toUtf8();
-            exitCode = CommandException;
+    /* Special arguments:
+     * "-"  read this argument from stdin
+     * "--" read all following arguments without control sequences
+     */
+    QScriptValueList fnArgs;
+    bool readRaw = false;
+    for (const auto &arg : args) {
+        if (readRaw) {
+            fnArgs.append( newByteArray(arg.toUtf8()) );
+        } else if (arg == "--") {
+            readRaw = true;
+        } else if (arg == "-") {
+            fnArgs.append( input() );
+        } else if (arg == "-e") {
+            fnArgs.append("eval");
         } else {
-            response = serializeScriptValue(result);
-            exitCode = CommandFinished;
+            const auto unescapedArg = parseCommandLineArgument(arg);
+            const auto value = newByteArray( unescapedArg.toUtf8() );
+            fnArgs.append(value);
         }
     }
 
-    if (exitCode == CommandFinished && hasData && actionData != m_data)
-        m_proxy->setActionData(m_actionId, data());
+    QString cmd;
+    QScriptValue result;
+
+    int skipArguments = 0;
+    while ( skipArguments < fnArgs.size() && canContinue() && !m_engine->hasUncaughtException() ) {
+        if ( result.isFunction() ) {
+            m_skipArguments = -1;
+            result = result.call( QScriptValue(), fnArgs.mid(skipArguments) );
+            if (m_skipArguments == -1)
+                break;
+            skipArguments += m_skipArguments;
+        } else {
+            cmd = toString(fnArgs[skipArguments], this);
+            result = eval(cmd);
+            ++skipArguments;
+        }
+    }
+
+    if ( result.isFunction() && canContinue() && !m_engine->hasUncaughtException() )
+        result = result.call( QScriptValue(), fnArgs.mid(skipArguments) );
+
+    int exitCode;
+
+    if (m_failed) {
+        exitCode = CommandError;
+    } else if (m_abort != Abort::None) {
+        exitCode = CommandFinished;
+    } else if ( m_engine->hasUncaughtException() ) {
+        const auto exceptionText = processUncaughtException(QString());
+        const auto message = createScriptErrorMessage(exceptionText).toUtf8();
+        printError(message);
+        exitCode = CommandException;
+    } else {
+        const auto message = serializeScriptValue(result);
+        print(message);
+        exitCode = CommandFinished;
+    }
+
+    if (exitCode == CommandFinished)
+        setActionData();
 
     // Destroy objects so destructors are run before script finishes
     // (e.g. file writes are flushed or temporary files are automatically removed).
     m_engine->collectGarbage();
 
-    sendMessageToClient(response, exitCode);
+    return exitCode;
 }
 
 void Scriptable::setInput(const QByteArray &input)
@@ -2576,24 +2496,44 @@ QString Scriptable::processUncaughtException(const QString &cmd)
 
     const auto exceptionText = exceptionName + backtraceText;
 
-    logScriptError(
-                QString("Exception in command \"%1\": %2")
-                .arg(cmd, exceptionText) );
+    auto label = m_actionName;
+    if ( !cmd.isEmpty() )
+        label.append("::" + cmd);
 
-    showExceptionMessage(exceptionName);
+    if (label.isEmpty()) {
+        logScriptError(
+            QString("Exception in command: %2")
+            .arg(exceptionText) );
+    } else {
+        logScriptError(
+            QString("Exception in command \"%1\": %2")
+            .arg(label, exceptionText) );
+    }
+
+    // Show exception popups only if the script was launched from application.
+    // (avoid it if launched from command line).
+    if (m_actionId != -1)
+        showExceptionMessage(exceptionName);
 
     return exceptionText;
 }
 
+void Scriptable::processUncaughtMonitorException(const char *label)
+{
+    if ( !engine()->hasUncaughtException() ) {
+        processUncaughtException(label);
+        engine()->clearExceptions();
+    }
+}
+
 void Scriptable::showExceptionMessage(const QString &message)
 {
     if (!m_proxy)
         return;
 
-    const auto actionName = getTextData( qgetenv("COPYQ_ACTION_NAME") );
-    const auto title = actionName.isEmpty()
+    const auto title = m_actionName.isEmpty()
         ? tr("Exception")
-        : tr("Exception in %1").arg( quoteString(actionName) );
+        : tr("Exception in %1").arg( quoteString(m_actionName) );
 
     const auto id = qHash(title) ^ qHash(message);
     const auto notificationId = QString::number(id);
@@ -2624,8 +2564,7 @@ QScriptValue Scriptable::copy(ClipboardMode mode)
         const QString mime = COPYQ_MIME_PREFIX "invalid";
         const QByteArray value = "invalid";
         data.insert(mime, value);
-        if ( !setClipboard(&data, mode) )
-            return false;
+        m_proxy->setClipboard(data, mode);
 
         m_proxy->copyFromCurrentWindow();
 
@@ -2643,7 +2582,8 @@ QScriptValue Scriptable::copy(ClipboardMode mode)
     if (args == 1) {
         QScriptValue value = argument(0);
         setTextData( &data, toString(value, this) );
-        return setClipboard(&data, mode);
+        m_proxy->setClipboard(data, mode);
+        return true;
     }
 
     if (args % 2 == 0) {
@@ -2655,40 +2595,26 @@ QScriptValue Scriptable::copy(ClipboardMode mode)
             toItemData(argument(++i), mime, &data);
         }
 
-        return setClipboard(&data, mode);
+        m_proxy->setClipboard(data, mode);
+        return true;
     }
 
     throwError(argumentError());
     return false;
 }
 
-bool Scriptable::setClipboard(QVariantMap *data, ClipboardMode mode)
-{
-    const QString mime = COPYQ_MIME_PREFIX "hash";
-    data->remove(mime);
-    const QByteArray id = QByteArray::number(hash(*data));
-    data->insert(mime, id);
-
-    for (int i = 0; i < setClipboardMaxRetries; ++i) {
-        m_proxy->setClipboard(*data, mode);
-
-        // Wait for clipboard to be set.
-        for (int j = 0; j < 10; ++j) {
-            if ( clipboardContains(mode, m_proxy, mime, id) )
-                return true;
-            waitFor(5 + i * 20 + j * 10);
-        }
-    }
-
-    throwError( tr("Failed to set clipboard!") );
-    return false;
-}
-
 void Scriptable::stopEventLoops()
 {
     emit stop();
 }
 
+void Scriptable::abortEvaluation(Abort abort)
+{
+    m_abort = abort;
+    throwError("Evaluation aborted");
+    emit finished();
+}
+
 void Scriptable::changeItem(bool create)
 {
     int row;
@@ -2736,9 +2662,9 @@ void Scriptable::nextToClipboard(int where)
     if (data.isEmpty())
         return;
 
-    setClipboard(&data, ClipboardMode::Clipboard);
+    m_proxy->setClipboard(data, ClipboardMode::Clipboard);
 #ifdef HAS_MOUSE_SELECTIONS
-    setClipboard(&data, ClipboardMode::Selection);
+    m_proxy->setClipboard(data, ClipboardMode::Selection);
 #endif
 }
 
@@ -2775,7 +2701,30 @@ QScriptValue Scriptable::eval(const QString &script, const QString &fileName)
         return QScriptValue();
     }
 
-    return engine()->evaluate(script, fileName);
+    const auto result = engine()->evaluate(script, fileName);
+
+    if (m_abort != Abort::None) {
+        engine()->clearExceptions();
+        if (m_abort == Abort::AllEvaluations)
+            abortEvaluation(Abort::AllEvaluations);
+        else
+            m_abort = Abort::None;
+
+        return QScriptValue();
+    }
+
+    return result;
+}
+
+void Scriptable::setActionId(int actionId)
+{
+    m_actionId = actionId;
+    getActionData();
+}
+
+void Scriptable::setActionName(const QString &actionName)
+{
+    m_actionName = actionName;
 }
 
 QScriptValue Scriptable::eval(const QString &script)
@@ -2799,6 +2748,40 @@ QTextCodec *Scriptable::codecFromNameOrThrow(const QScriptValue &codecName)
 
 bool Scriptable::runAction(Action *action)
 {
+    if (!canContinue())
+        return false;
+
+    // Shortcut to run script in current Scriptable
+    // instead of spawning new process.
+    const auto &cmd = action->command();
+    const auto cmd1 = cmd.value(0).value(0);
+    if ( cmd.size() == 1 && cmd[0].size() == 1
+         && cmd1.size() >= 2
+         && cmd1[0] == "copyq"
+         && (!cmd1[1].startsWith("-") || cmd1[1] == "-e") )
+    {
+        const auto oldInput = m_input;
+        m_input = newByteArray(action->input());
+
+        const auto oldAction = m_action;
+        m_action = action;
+        engine()->pushContext();
+
+        const auto exitCode = executeArguments(cmd1.mid(1));
+        action->setExitCode(exitCode);
+        m_failed = false;
+        m_engine->clearExceptions();
+
+        engine()->popContext();
+        m_action = oldAction;
+        m_input = oldInput;
+
+        return true;
+    }
+
+    // Update data for the new action.
+    setActionData();
+
     action->setWorkingDirectory( m_dirClass->getCurrentPath() );
     action->start();
 
@@ -2807,13 +2790,16 @@ bool Scriptable::runAction(Action *action)
         return false;
     }
 
-    while ( !action->waitForFinished(5000) && m_connected ) {}
+    while ( !action->waitForFinished(5000) && canContinue() ) {}
 
     if ( action->isRunning() && !action->waitForFinished(5000) ) {
         action->terminate();
         return false;
     }
 
+    // Data could have changed so refetch them.
+    getActionData();
+
     return true;
 }
 
@@ -2837,10 +2823,10 @@ bool Scriptable::runCommands(CommandType::CommandType type)
         if ( !canExecuteCommand(command) )
             continue;
 
-        if ( m_connected && !command.cmd.isEmpty() ) {
+        if ( canContinue() && !command.cmd.isEmpty() ) {
             Action action;
             action.setCommand( command.cmd, QStringList(getTextData(m_data)) );
-            action.setInput(m_data, command.input);
+            action.setInputWithFormat(m_data, command.input);
             action.setName(command.name);
             action.setData(m_data);
 
@@ -2849,19 +2835,17 @@ bool Scriptable::runCommands(CommandType::CommandType type)
             //action.setItemSeparator(QRegExp(command.sep));
             //action.setOutputTab(command.outputTab);
 
-            if ( !runAction(&action) && m_connected ) {
+            if ( !runAction(&action) && canContinue() ) {
                 throwError( QString(label).arg(command.name, "Failed to start") );
                 return false;
             }
         }
 
-        if (!m_connected) {
+        if (!canContinue()) {
             COPYQ_LOG( QString(label).arg(command.name, "Interrupted") );
             return false;
         }
 
-        m_data = m_proxy->getActionData(m_actionId);
-
         if ( type == CommandType::Automatic ) {
             if ( !command.tab.isEmpty() ) {
                 auto data = copyWithoutInternalData(m_data);
@@ -2874,7 +2858,7 @@ bool Scriptable::runCommands(CommandType::CommandType type)
             }
         }
 
-        COPYQ_LOG( QString(label).arg(command.name, "Finished") );
+        COPYQ_LOG_VERBOSE( QString(label).arg(command.name, "Finished") );
     }
 
     return true;
@@ -2923,7 +2907,7 @@ bool Scriptable::canExecuteCommandFilter(const QString &matchCommand)
 
 bool Scriptable::verifyClipboardAccess()
 {
-    if ( qobject_cast<QApplication*>(qApp) != nullptr )
+    if ( qobject_cast<QGuiApplication*>(qApp) != nullptr )
         return true;
 
     throwError("Cannot access system clipboard with QCoreApplication");
@@ -2935,23 +2919,35 @@ void Scriptable::provideClipboard(ClipboardMode mode)
     if (!verifyClipboardAccess())
         return;
 
-    auto clipboard = createPlatformNativeInterface()->clipboard();
-    clipboard->setData(mode, m_data);
+    const auto owner = makeClipboardOwnerData();
+    m_data.insert(mimeOwner, owner);
 
-    const auto slot = mode == ClipboardMode::Clipboard
-            ? &Scriptable::onProvidedClipboardChanged
-            : &Scriptable::onProvidedSelectionChanged;
+    createPlatformNativeInterface()->clipboard()->setData(mode, m_data);
+
+    QEventLoop loop;
+
+    const auto checkClipboardOwnership = [&]() {
+        if ( clipboardOwnerData(mode) != owner )
+            loop.quit();
+    };
 
     QTimer t;
     t.setInterval(8000);
-    connect(&t, &QTimer::timeout, this, slot);
+    connect(&t, &QTimer::timeout, this, checkClipboardOwnership);
     t.start();
 
-    QEventLoop loop;
     connect( this, &Scriptable::finished, &loop, &QEventLoop::quit );
     connect( this, &Scriptable::stop, &loop, &QEventLoop::quit );
-    connect( clipboard.get(), &PlatformClipboard::changed, this, slot );
+    connect( QGuiApplication::clipboard(), &QClipboard::changed,
+             &t, checkClipboardOwnership );
+
+    COPYQ_LOG( QString("Started providing %1")
+               .arg(mode == ClipboardMode::Clipboard ? "clipboard" : "selection") );
+
     loop.exec();
+
+    COPYQ_LOG( QString("Finished providing %1")
+               .arg(mode == ClipboardMode::Clipboard ? "clipboard" : "selection") );
 }
 
 void Scriptable::insert(int argumentsEnd)
@@ -2996,6 +2992,101 @@ QStringList Scriptable::arguments()
     return args;
 }
 
+void Scriptable::print(const QByteArray &message)
+{
+    if (m_action) {
+        m_action->appendOutput(message);
+    } else {
+        QFile f;
+        f.open(stdout, QIODevice::WriteOnly);
+        f.write(message);
+    }
+}
+
+void Scriptable::printError(const QByteArray &message)
+{
+    if (m_action) {
+        m_action->appendErrorOutput(message);
+    } else {
+        QFile f;
+        f.open(stderr, QIODevice::WriteOnly);
+        f.write(message);
+        if ( !message.endsWith('\n') )
+            f.write("\n");
+    }
+}
+
+void Scriptable::getActionData()
+{
+    if (m_actionId != -1)
+        m_data = m_oldData = m_proxy->getActionData(m_actionId);
+}
+
+void Scriptable::setActionData()
+{
+    if (m_actionId != -1 && m_oldData != m_data) {
+        m_proxy->setActionData(m_actionId, m_data);
+        m_oldData = m_data;
+    }
+}
+
+void Scriptable::synchronizeSelection(ClipboardMode targetMode)
+{
+#ifdef HAS_MOUSE_SELECTIONS
+#   define COPYQ_SYNC_LOG(MESSAGE) \
+        COPYQ_LOG( QString("Synchronizing to %1: " MESSAGE) \
+                   .arg(targetMode == ClipboardMode::Clipboard ? "clipboard" : "selection") );
+
+    if (!verifyClipboardAccess())
+        return;
+
+    {
+        // Avoid changing clipboard after a text is selected just before it's copied
+        // with a keyboard shortcut.
+        SleepTimer t(5000);
+        while ( QGuiApplication::queryKeyboardModifiers() != Qt::NoModifier ) {
+            if ( !t.sleep() && !canContinue() )
+                return;
+        }
+
+        const auto target = targetMode == ClipboardMode::Clipboard ? QClipboard::Clipboard : QClipboard::Selection;
+        const auto source = targetMode == ClipboardMode::Selection ? QClipboard::Clipboard : QClipboard::Selection;
+
+        // Stop if the clipboard/selection text already changed again.
+        auto clipboard = QGuiApplication::clipboard();
+        const auto sourceText = clipboard->text(source);
+        if (sourceText != getTextData(m_data)) {
+            COPYQ_SYNC_LOG("Cancelled (source text changed)");
+            return;
+        }
+
+        const auto targetText = clipboard->text(target);
+        const auto targetTextHash = m_data.value(COPYQ_MIME_PREFIX "target-text-hash").toByteArray().toUInt();
+        if (targetTextHash != qHash(targetText)) {
+            COPYQ_SYNC_LOG("Cancelled (target text changed)");
+            return;
+        }
+
+        // Stop if the clipboard and selection text is already synchronized
+        // or user selected text and copied it to clipboard.
+        if (sourceText == targetText) {
+            COPYQ_SYNC_LOG("Cancelled (target is same as source)");
+            return;
+        }
+    }
+
+    COPYQ_SYNC_LOG("Calling provideClipboard()/provideSelection()");
+
+    if (targetMode == ClipboardMode::Clipboard)
+        provideClipboard();
+    else
+        provideSelection();
+#   undef COPYQ_SYNC_LOG
+#else
+    Q_UNUSED(targetMode);
+#endif
+}
+
 QScriptValue NetworkReply::get(const QString &url, Scriptable *scriptable)
 {
     NetworkReply *reply = new NetworkReply(url, QByteArray(), scriptable);
