@@ -46,6 +46,8 @@ class QNetworkAccessManager;
 class QScriptEngine;
 class QTextCodec;
 
+enum class ClipboardOwnership;
+
 class Scriptable : public QObject, protected QScriptable
 {
     Q_OBJECT
@@ -65,8 +67,6 @@ class Scriptable : public QObject, protected QScriptable
     Q_PROPERTY(QScriptValue mimeShortcut READ getMimeShortcut)
     Q_PROPERTY(QScriptValue mimeColor READ getMimeColor)
     Q_PROPERTY(QScriptValue mimeOutputTab READ getMimeOutputTab)
-    Q_PROPERTY(QScriptValue mimeSyncToClipboard READ getMimeSyncToClipboard)
-    Q_PROPERTY(QScriptValue mimeSyncToSelection READ getMimeSyncToSelection)
 
     Q_PROPERTY(QScriptValue global READ getGlobal)
     Q_PROPERTY(QScriptValue plugins READ getPlugins)
@@ -77,6 +77,12 @@ public:
             ScriptableProxy *proxy,
             QObject *parent = nullptr);
 
+    enum class Abort {
+        None,
+        CurrentEvaluation,
+        AllEvaluations,
+    };
+
     // WORKAROUND: These methods override the one in QScriptable,
     //             which doesn't work well in some cases.
     QScriptContext *context() const;
@@ -116,13 +122,9 @@ public:
     void throwSaveError(const QString &filePath);
     void throwImportError(const QString &filePath);
 
-    void sendMessageToClient(const QByteArray &message, int exitCode);
-
     QScriptEngine *engine() const { return m_engine; }
 
-    bool isConnected() const { return m_connected; }
-
-    const QVariantMap &data() const { return m_data; }
+    bool canContinue() const { return m_abort == Abort::None && !m_failed; }
 
     QScriptValue getMimeText() const { return mimeText; }
     QScriptValue getMimeHtml() const { return mimeHtml; }
@@ -139,8 +141,6 @@ public:
     QScriptValue getMimeShortcut() const { return mimeShortcut; }
     QScriptValue getMimeColor() const { return mimeColor; }
     QScriptValue getMimeOutputTab() const { return mimeOutputTab; }
-    QScriptValue getMimeSyncToClipboard() const { return mimeSyncToClipboard; }
-    QScriptValue getMimeSyncToSelection() const { return mimeSyncToSelection; }
 
     QScriptValue getGlobal();
     QScriptValue getPlugins();
@@ -153,12 +153,14 @@ public:
 
     QScriptValue eval(const QString &script, const QString &fileName);
 
-    void executeArguments(const QStringList &args);
-
-    bool setClipboard(QVariantMap *data, ClipboardMode mode);
+    void setActionId(int actionId);
+    void setActionName(const QString &actionName);
+    int executeArguments(const QStringList &args);
 
     void stopEventLoops();
 
+    void abortEvaluation(Abort abort = Abort::AllEvaluations);
+
 public slots:
     void setInput(const QByteArray &input);
 
@@ -337,10 +339,12 @@ public slots:
     void onOwnClipboardChanged();
     void onHiddenClipboardChanged();
 
+    void synchronizeToSelection();
+    void synchronizeFromSelection();
+
     void setClipboardData();
     void updateTitle();
     void setTitle();
-    void synchronizeSelection();
     void saveData();
     QScriptValue hasData();
     void showDataNotification();
@@ -358,23 +362,20 @@ public slots:
     void provideSelection();
 
 signals:
-    void sendMessage(const QByteArray &message, int messageCode);
     void dataReceived();
     void finished();
     void readInput();
-    void resetSynchronizeSelectionTimer();
-    void startSynchronizeSelectionTimer(ClipboardMode targetMode, const QVariantMap &data);
     void stop();
 
 private:
     void onExecuteOutput(const QByteArray &output);
-    void onMonitorRunScriptRequest(const QString &script, const QVariantMap &data);
-    void onProvidedClipboardChanged();
-    void onProvidedSelectionChanged();
+    void onMonitorClipboardChanged(const QVariantMap &data, ClipboardOwnership ownership);
+    void onSynchronizeSelection(ClipboardMode sourceMode, const QString &text, uint targetTextHash);
 
     bool sourceScriptCommands();
     void callDisplayFunctions(QScriptValueList displayFunctions);
     QString processUncaughtException(const QString &cmd);
+    void processUncaughtMonitorException(const char *label);
     void showExceptionMessage(const QString &message);
     QVector<int> getRows() const;
     QScriptValue copy(ClipboardMode mode);
@@ -396,6 +397,14 @@ private:
 
     QStringList arguments();
 
+    void print(const QByteArray &message);
+    void printError(const QByteArray &message);
+
+    void getActionData();
+    void setActionData();
+
+    void synchronizeSelection(ClipboardMode targetMode);
+
     ScriptableProxy *m_proxy;
     QScriptEngine *m_engine;
     ByteArrayClass *m_baClass;
@@ -405,8 +414,10 @@ private:
     QString m_inputSeparator;
     QScriptValue m_input;
     QVariantMap m_data;
+    QVariantMap m_oldData;
     int m_actionId = -1;
-    bool m_connected;
+    QString m_actionName;
+    Abort m_abort = Abort::None;
     int m_skipArguments = 0;
 
     // FIXME: Parameters for execute() shouldn't be global.
@@ -417,6 +428,9 @@ private:
     bool m_displayFunctionsLock = false;
 
     QScriptValue m_plugins;
+
+    Action *m_action = nullptr;
+    bool m_failed = false;
 };
 
 class NetworkReply : public QObject {
