@@ -19,6 +19,7 @@
 
 #include "scriptableproxy.h"
 
+#include "common/action.h"
 #include "common/appconfig.h"
 #include "common/command.h"
 #include "common/commandstore.h"
@@ -89,7 +90,8 @@ const quint32 serializedFunctionCallVersion = 1;
 #define STR(str) str
 
 #define INVOKE_(function, arguments) \
-    static auto f = FunctionCallSerializer(m_tabName, STR(#function)).withSlotArguments arguments; \
+    static auto f = FunctionCallSerializer(STR(#function)).withSlotArguments arguments; \
+    f.setTabName(m_tabName); \
     f.setArguments arguments; \
     emit sendFunctionCall(f.serializeAndClear()) \
 
@@ -286,9 +288,9 @@ public:
 
 class FunctionCallSerializer {
 public:
-    explicit FunctionCallSerializer(const QString &tabName, const char *functionName)
+    explicit FunctionCallSerializer(const char *functionName)
     {
-        m_args << tabName << QByteArray(functionName);
+        m_args << QVariant() << QByteArray(functionName);
     }
 
     template<typename ...Ts>
@@ -319,6 +321,11 @@ public:
         setArguments(args...);
     }
 
+    void setTabName(const QString &tabName)
+    {
+        m_args[0] = tabName;
+    }
+
 private:
     void setSlotArgumentTypes(const QByteArray &args)
     {
@@ -841,7 +848,7 @@ void ScriptableProxy::disableMonitoring(bool arg1)
 void ScriptableProxy::setClipboard(const QVariantMap &data, ClipboardMode mode)
 {
     INVOKE2(setClipboard, (data, mode));
-    m_wnd->setClipboardAndWait(data, mode);
+    m_wnd->setClipboard(data, mode);
 }
 
 QString ScriptableProxy::renameTab(const QString &arg1, const QString &arg2)
@@ -918,6 +925,15 @@ void ScriptableProxy::action(const QVariantMap &arg1, const Command &arg2)
     m_wnd->action(arg1, arg2, QModelIndex());
 }
 
+void ScriptableProxy::runInternalAction(const QVariantMap &data, const QString &command)
+{
+    INVOKE2(runInternalAction, (data, command));
+    auto action = new Action();
+    action->setCommand(command);
+    action->setData(data);
+    m_wnd->runInternalAction(action);
+}
+
 void ScriptableProxy::showMessage(const QString &title,
         const QString &msg,
         const QString &icon,
@@ -952,12 +968,11 @@ QVariantMap ScriptableProxy::nextItem(int where)
     return c->copyIndex(index);
 }
 
-void ScriptableProxy::browserMoveToClipboard(int arg1)
+void ScriptableProxy::browserMoveToClipboard(int row)
 {
-    INVOKE2(browserMoveToClipboard, (arg1));
+    INVOKE2(browserMoveToClipboard, (row));
     ClipboardBrowser *c = fetchBrowser();
-    if (c)
-        c->moveToClipboard(c->index(arg1));
+    m_wnd->moveToClipboard(c, row);
 }
 
 void ScriptableProxy::browserSetCurrent(int arg1)
@@ -1622,14 +1637,16 @@ QString ScriptableProxy::translationsPath()
 QString ScriptableProxy::iconColor()
 {
     INVOKE(iconColor, ());
-    return m_wnd->sessionIconColor().name();
+    const auto color = m_wnd->sessionIconColor();
+    return color.isValid() ? color.name() : QString();
 }
 
 bool ScriptableProxy::setIconColor(const QString &colorName)
 {
     INVOKE(setIconColor, (colorName));
+
     QColor color(colorName);
-    if ( !color.isValid() )
+    if ( !colorName.isEmpty() && !color.isValid() )
         return false;
 
     m_wnd->setSessionIconColor(color);
