@@ -119,6 +119,8 @@ public slots:
 
     void action(const QVariantMap &arg1, const Command &arg2);
 
+    void runInternalAction(const QVariantMap &data, const QString &command);
+
     void showMessage(const QString &title,
             const QString &msg,
             const QString &icon,
@@ -127,7 +129,7 @@ public slots:
             const NotificationButtons &buttons = NotificationButtons());
 
     QVariantMap nextItem(int where);
-    void browserMoveToClipboard(int arg1);
+    void browserMoveToClipboard(int row);
     void browserSetCurrent(int arg1);
     QString browserRemoveRows(QVector<int> rows);
 
