@@ -63,6 +63,13 @@ do { \
 #define WAIT_FOR_CLIPBOARD2(DATA, MIME) \
     QCOMPARE( waitUntilClipboardSet(DATA, MIME), QByteArray(DATA) )
 
+#define RETURN_ON_ERROR(CALLBACK, ERROR) \
+    do { \
+        const auto errors = (CALLBACK); \
+        if ( !errors.isEmpty() ) \
+            return QByteArray(ERROR) + ":\n" + QByteArray(errors); \
+    } while(false)
+
 /// Skip rest of the tests
 #define SKIP(MESSAGE) QSKIP(MESSAGE, SkipAll)
 
@@ -88,6 +95,10 @@ do { \
     QCOMPARE(QString(out_), QString(expected_)); \
 } while(false)
 
+#define SKIP_ON_ENV(ENV) \
+    if ( qgetenv(ENV) == "1" ) \
+        SKIP("Unset " ENV " to run the tests")
+
 /// Interval to wait (in ms) until window is shown and focused.
 const int waitMsShow = 1000;
 
@@ -100,6 +111,9 @@ const int waitMsSetClipboard = 1000;
 /// Interval to wait (in ms) before getting clipboard.
 const int waitMsGetClipboard = 100;
 
+/// Interval to wait (in ms) for pasting clipboard.
+const int waitMsPasteClipboard = 1000;
+
 /// Interval to wait (in ms) for client process.
 const int waitClientRun = 30000;
 
