@@ -250,6 +250,12 @@ public:
     int run(const QStringList &arguments, QByteArray *stdoutData = nullptr,
             QByteArray *stderrData = nullptr, const QByteArray &in = QByteArray()) override
     {
+        if (stdoutData != nullptr)
+            stdoutData->clear();
+
+        if (stderrData != nullptr)
+            stderrData->clear();
+
         QProcess p;
         if (!startTestProcess(&p, arguments))
             return -1;
@@ -261,13 +267,9 @@ public:
 
         if (stdoutData == nullptr)
             p.closeReadChannel(QProcess::StandardOutput);
-        else
-            stdoutData->clear();
 
         if (stderrData == nullptr)
             p.closeReadChannel(QProcess::StandardError);
-        else
-            stderrData->clear();
 
         SleepTimer t(waitClientRun);
         while ( p.state() == QProcess::Running ) {
@@ -699,6 +701,8 @@ void Tests::commandPrint()
 void Tests::commandAbort()
 {
     RUN("eval" << "abort(); 1", "");
+    RUN("eval" << "eval('abort(); print(1)'); 2", "");
+    RUN("eval" << "execute('copyq', 'eval', '--', 'abort(); print(1)'); 2", "2\n");
 }
 
 void Tests::commandFail()
@@ -1283,6 +1287,25 @@ void Tests::commandScreenshot()
     RUN("screenshot().size() > 0", "true\n")
 }
 
+void Tests::commandNotification()
+{
+    const auto script = R"(
+        notification(
+            '.title', 'title',
+            '.message', 'message',
+            '.time', 1000,
+            '.id', 'test',
+            '.icon', 'copyq',
+            '.button', 'OK', '', '',
+            '.button', 'CANCEL', '', ''
+        )
+        )";
+    RUN(script, "");
+
+    RUN_EXPECT_ERROR_WITH_STDERR(
+                "notification('.message', 'message', 'BAD')", CommandException, "Unknown argument: BAD");
+}
+
 void Tests::commandIcon()
 {
     RUN("iconColor", QByteArray(defaultSessionColor) + "\n");
@@ -1293,6 +1316,9 @@ void Tests::commandIcon()
     RUN_EXPECT_ERROR("iconColor" << "BAD_COLOR_NAME", CommandException);
     RUN("iconColor", "#ff0000\n");
 
+    RUN("iconColor" << "", "");
+    RUN("iconColor", QByteArray(defaultSessionColor) + "\n");
+
     RUN("iconColor" << defaultSessionColor, "");
     RUN("iconColor", QByteArray(defaultSessionColor) + "\n");
 }
@@ -1322,13 +1348,25 @@ void Tests::commandIconTagColor()
     RUN("iconTagColor", QByteArray(defaultTagColor) + "\n");
 }
 
+void Tests::commandDateString()
+{
+    const auto dateFormat = "TEST:yyyy-MM-dd";
+    const auto dateTime = QDateTime::currentDateTime();
+    const auto today = dateTime.toString(dateFormat);
+    RUN("dateString" << dateFormat, today + "\n");
+}
+
+void Tests::commandAfterMilliseconds()
+{
+    const QString script = "afterMilliseconds(100, function(){ print('TEST'); abort(); });";
+    RUN(script, "");
+    RUN(script + "sleep(1)", "");
+    RUN(script + "sleep(200)", "TEST");
+}
+
 void Tests::classFile()
 {
-    RUN("eval" <<
-        "; var f = new File('/missing.file')"
-        "; f.exists()"
-        , "false\n"
-        );
+    RUN("var f = new File('/copyq_missing_file'); f.exists()", "false\n");
 }
 
 void Tests::classDir()
@@ -1344,26 +1382,94 @@ void Tests::classDir()
 
 void Tests::classTemporaryFile()
 {
-    RUN("eval" << "var f = new TemporaryFile(); f.open()", "true\n");
+    RUN("var f = new TemporaryFile(); f.open()", "true\n");
+
+    QByteArray err;
 
     for ( const auto autoRemove : {true, false} ) {
-        QByteArray out;
-        const auto cmd =
+        QByteArray fileName;
+        const auto script =
                 QString(R"(
                         var f = new TemporaryFile()
                         if (!f.open())
                             throw 'Failed to open temporary file'
+
                         f.setAutoRemove(%1)
                         print(f.fileName())
                         )").arg(autoRemove);
-        run( Args("eval") << cmd, &out );
+        run(Args() << script, &fileName, &err);
+        QVERIFY2( testStderr(err), err );
 
-        QFile f( QString::fromUtf8(out) );
+        QFile f( QString::fromUtf8(fileName) );
         QVERIFY( f.exists() != autoRemove );
 
         if (!autoRemove)
             f.remove();
     }
+
+    QByteArray fileName;
+    const auto scriptWrite = R"(
+        var f = new TemporaryFile()
+        if (!f.open())
+            throw 'Failed to open temporary file'
+
+        if (!f.write('LINE'))
+            throw 'Failed to write to temporary file'
+
+        f.setAutoRemove(false)
+        print(f.fileName())
+        )";
+    run(Args() << scriptWrite, &fileName);
+    QVERIFY2( testStderr(err), err );
+    QVERIFY( QFile::exists(QString::fromUtf8(fileName)) );
+
+    QByteArray out;
+    const auto scriptRead = R"(
+        var f = new File(str(input()))
+        if (!f.openReadOnly())
+            throw 'Failed to open file'
+
+        print(''
+            + ' exists()=' + str(f.exists())
+            + ' isOpen()=' + str(f.isOpen())
+            + ' isReadable()=' + str(f.isReadable())
+            + ' isWritable()=' + str(f.isWritable())
+            + ' size()=' + str(f.size())
+            + ' readAll()=' + str(f.readAll())
+            + ' atEnd()=' + str(f.atEnd())
+            + ' seek(0)=' + str(f.seek(0))
+            + ' read(1)=' + str(f.read(1))
+            + ' pos()=' + str(f.pos())
+            + ' peek(1)=' + str(f.peek(1))
+            + ' readLine()=' + str(f.readLine())
+        )
+        )";
+    const QByteArray expectedOut =
+        " exists()=true"
+        " isOpen()=true"
+        " isReadable()=true"
+        " isWritable()=false"
+        " size()=4"
+        " readAll()=LINE"
+        " atEnd()=true"
+        " seek(0)=true"
+        " read(1)=L"
+        " pos()=1"
+        " peek(1)=I"
+        " readLine()=INE";
+    run(Args() << scriptRead, &out, &err, fileName);
+    QVERIFY2( testStderr(err), err );
+    QCOMPARE(out, expectedOut);
+
+    const auto scriptRemove = R"(
+        var f = new File(str(input()))
+        if (!f.remove())
+            throw 'Failed to remove file'
+        )";
+    run(Args() << scriptRemove, &out, &err, fileName);
+    QVERIFY2( testStderr(err), err );
+    QCOMPARE(QByteArray(), out);
+    QVERIFY( !QFile::exists(QString::fromUtf8(fileName)) );
 }
 
 void Tests::chainingCommands()
@@ -2364,6 +2470,27 @@ void Tests::shortcutCommandSetSelectedItemsData()
     RUN("read" << "2", "Y");
 }
 
+void Tests::shortcutCommandSelectedAndCurrent()
+{
+    const auto script = R"(
+        setCommands([{
+            name: 'Set Data for Second Selected Item',
+            inMenu: true,
+            shortcuts: ['Ctrl+F1'],
+            output: 'text/plain',
+            cmd: 'copyq: print(selectedItems() + "|" + currentItem() + "|" + selectedTab())'
+        }])
+        )";
+    RUN(script, "");
+
+    const auto tab1 = testTab(1);
+    RUN("tab" << tab1 << "add" << "C" << "B" << "A", "");
+
+    RUN("tab" << tab1 << "setCurrentTab" << tab1 << "selectItems" << "1" << "2", "true\n");
+    RUN("keys" << "CTRL+F1", "");
+    RUN("tab" << tab1 << "read(0)", "1,2|2|" + tab1);
+}
+
 void Tests::automaticCommandIgnore()
 {
     const auto script = R"(
@@ -2570,6 +2697,21 @@ void Tests::queryKeyboardModifiersCommand()
     // TODO: Is there a way to press modifiers?
 }
 
+void Tests::setTabName()
+{
+    const auto script = R"(
+        tab('1')
+        add(1)
+        tab1Size = size()
+
+        tab('2')
+        tab2Size = size()
+
+        print(tab1Size + ',' + tab2Size)
+        )";
+    RUN(script, "1,0");
+}
+
 int Tests::run(const QStringList &arguments, QByteArray *stdoutData, QByteArray *stderrData, const QByteArray &in)
 {
     return m_test->run(arguments, stdoutData, stderrData, in);
