@@ -114,10 +114,16 @@ private slots:
 
     void commandScreenshot();
 
+    void commandNotification();
+
     void commandIcon();
     void commandIconTag();
     void commandIconTagColor();
 
+    void commandDateString();
+
+    void commandAfterMilliseconds();
+
     void classFile();
     void classDir();
     void classTemporaryFile();
@@ -188,6 +194,7 @@ private slots:
     void shortcutCommandSetSelectedItemData();
     void shortcutCommandSelectedItemsData();
     void shortcutCommandSetSelectedItemsData();
+    void shortcutCommandSelectedAndCurrent();
 
     void automaticCommandIgnore();
     void automaticCommandRemove();
@@ -206,6 +213,8 @@ private slots:
 
     void queryKeyboardModifiersCommand();
 
+    void setTabName();
+
 private:
     void clearServerErrors();
     int run(const QStringList &arguments, QByteArray *stdoutData = nullptr,
